# Lets the user play a simple game of Hangman using a supplied dictionary.
# Copyright 2011 Nathaniel Guy.

import random
import os

# Draws hangman ASCII art.
#
# Arguments:
#    num_missed: Number of missed guesses
#    misses_string: String containing wrong letters guessed so far
#
# Return values:
#    none
#
def draw_hangman(num_missed, misses_string):
    # num_missed should always be between 0 and 6, inclusive,
    # so fail if it isn't
    assert ((num_missed >= 0) and (num_missed <= 6))

    # Pad string with spaces so it's 15 characters long
    padded_string = misses_string.ljust(15, ' ')
    
    if num_missed is 0:
        # Draw hangman's noose with no hangman
        print "========================="
        print "|           |           |"
        print "|           |           |"
        print "|                       |"
        print "|                       |"      
        print "|                       |"        
        print "|                       |"
        print "|                       |"
        print "|                       |"
        print "|                       |"
        print "|                       |"
        print "|                       |"
        print "|                       |"
        print "|   |===============|   |"
        print "|   |%s|   |" % padded_string
        print "========================="
        
    elif num_missed is 1:
        # Draw hangman's noose with head
        print "========================="
        print "|           |           |"
        print "|           |           |"
        print "|          - -          |"
        print "|        | o o |        |"      
        print "|          ---          |"        
        print "|                       |"
        print "|                       |"
        print "|                       |"
        print "|                       |"
        print "|                       |"
        print "|                       |"
        print "|                       |"
        print "|   |===============|   |"
        print "|   |%s|   |" % padded_string
        print "========================="   
        
    elif num_missed is 2:
        # Draw hangman's noose with head and torso
        print "========================="
        print "|           |           |"
        print "|           |           |"
        print "|          - -          |"
        print "|        | o o |        |"      
        print "|          ---          |"        
        print "|           |           |"
        print "|           |           |"
        print "|           |           |"
        print "|           |           |"
        print "|                       |"
        print "|                       |"
        print "|                       |"
        print "|   |===============|   |"
        print "|   |%s|   |" % padded_string
        print "========================="   
        
    elif num_missed is 3:
        # Draw hangman's noose with head, torso and left arm
        print "========================="
        print "|           |           |"
        print "|           |           |"
        print "|          - -          |"
        print "|        | o o |        |"      
        print "|          ---          |"        
        print "|           |           |"
        print "|       ----|           |"
        print "|           |           |"
        print "|           |           |"
        print "|                       |"
        print "|                       |"
        print "|                       |"
        print "|   |===============|   |"
        print "|   |%s|   |" % padded_string
        print "========================="   
        
    elif num_missed is 4:
        # Draw hangman's noose with head, torso and both arms
        print "========================="
        print "|           |           |"
        print "|           |           |"
        print "|          - -          |"
        print "|        | o o |        |"      
        print "|          ---          |"        
        print "|           |           |"
        print "|       ----|----       |"
        print "|           |           |"
        print "|           |           |"
        print "|                       |"
        print "|                       |"
        print "|                       |"
        print "|   |===============|   |"
        print "|   |%s|   |" % padded_string
        print "========================="   
        
    elif num_missed is 5:
        # Draw hangman's noose with head, torso, arms, and left leg
        print "========================="
        print "|           |           |"
        print "|           |           |"
        print "|          - -          |"
        print "|        | o o |        |"      
        print "|          ---          |"        
        print "|           |           |"
        print "|       ----|----       |"
        print "|           |           |"
        print "|           |           |"
        print "|          /            |"
        print "|         /             |"
        print "|                       |"
        print "|   |===============|   |"
        print "|   |%s|   |" % padded_string
        print "========================="   
        
    elif num_missed is 6:
        # Draw hangman's noose with head, torso, arms, and both legs
        print "========================="
        print "|           |           |"
        print "|           |           |"
        print "|          - -          |"
        print "|        | x x |        |"      
        print "|          ---          |"        
        print "|           |           |"
        print "|       ----|----       |"
        print "|           |           |"
        print "|           |           |"
        print "|          / \          |"
        print "|         /   \         |"
        print "|                       |"
        print "|   |===============|   |"
        print "|   |%s|   |" % padded_string
        print "========================="   


# Prints the word to guess, with blanks where the user hasn't made
# a successful guess yet.
#
# Arguments:
#    word: Word to guess
#    guesses_string: String containing letters guessed so far
#
# Return values:
#    none
#
def print_word_with_blanks(word, guesses_string):
    for char in word:
        if char in guesses_string:
            print '%s ' % char,
        else:
            print '_ ',
    print '\n'

# Clears the screen, based on whether the user is in a Windows environment
# (where the command is "cls") or a Unix/Mac environment (where the command
# is "clear")
#
# Arguments:
#    none
#
# Return values:
#    none
#    
def clear_screen():
    os.system('cls' if os.name == 'nt' else 'clear')
    
# Main function
def main():
    # Load dictionary file
    words = open('dictionary.txt', 'r').readlines()
    word_count = len(words)
        
    # Loop until the user says to stop
    still_playing = True
    score = 0
    round = 0
    
    # Main Game Loop (one loop iteration per round)
    while(still_playing):
         # Pick a random word in the giant list of words, and make it uppercase
        word = words[random.randint(0, word_count - 1)].upper().strip()

        # Set number of wrong guesses to 0
        num_misses = 0
        misses_string = ''
        guesses_string = ''
        
        in_round = True
        # Main Round Loop (one loop iteration per word input)
        while(in_round):
            clear_screen()

            # Check if the player won by comparing word to guessed letters
            won_round = True
            for char in word:
                if char not in guesses_string:
                    won_round = False
            
            draw_hangman(num_misses, misses_string)

            # If we just won, increment the score before printing it
            if(won_round):
                score += 1
            print 'Score: %i\n' % score
            
            if(won_round):
                print "You win! Great job! The word was %s." % word
                in_round = False
            else:
                # If the user hasn't exceeded their guess limit
                if(num_misses < 6):
                    # Print word with guesses filled in
                    print_word_with_blanks(word, guesses_string)
                    # Loop until a valid guess has been obtained
                    while(True):
                        # Grab user input
                        input = raw_input('Guess a letter: ').upper()
                        # Extract first letter from input, if possible
                        if input is not '':
                            guess = input[0]
                        else:
                            guess = ''
                        # Reject non-alphabetic guesses
                        if not guess.isalpha():
                            print "Not a valid guess! Try again."
                        # Reject guesses that have already been guesses
                        elif guess in guesses_string:
                            print "Already used that guess! Try again."
                        # If we've gotten this far, we can use the guess!
                        else:
                            guesses_string += guess
                            if(guess in word):
                                print '%s is in the word!' % guess
                            else:
                                print '%s is not in the word!' % guess
                                num_misses += 1
                                misses_string += guess + ' '
                            # We have a valid guess, so let's break the loop
                            break
                # If the user has exceeded the guess limit, they lose the round
                else:
                    # Print full word
                    print_word_with_blanks(word, word)
                    print "Sorry, you lost... the word was %s." % word
                    # Round will be left at the completion of current loop
                    in_round = False
            raw_input('Press enter to continue...\n')
        # Round has ended
        round += 1
        answer = raw_input("You've won %i out of %i rounds. Play again (default: yes)? " % (score, round))
        # Check various ways the user could say yes to start a new round
        if (answer.lower() == 'yes') or (answer.lower() == 'y') or \
            (answer.lower() == 'sure') or (answer.lower() == 'yeah') or \
            (answer == ''):
            
            still_playing = True
        else:
            still_playing = False
    
    # Game is over
    print "Thanks for playing!"

# Load main function when script is run
if __name__ == "__main__":
    main()